"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.compile = exports.run = exports.parse = void 0;
const parser_1 = require("./parser");
const runner_1 = require("./runner");
const utils_1 = require("./utils");
/**
 * Bigodon class, used if you need custom settings or helpers
 * not present in the default functions.
 */
class Bigodon {
    constructor() {
        this.helpers = new Map();
        /**
         * Parses a template and returns an AST representing it.
         * This can be persisted as JSON for later usage.
         *
         * @param {string} template Bigodon template to be parsed
         * @return {TemplateStatement} AST representing input template
         */
        this.parse = (template) => {
            (0, utils_1.ensure)(typeof template === 'string', 'Template must be a string');
            return parser_1.$template.parse(template);
        };
        /**
         * Runs an AST returned by the {@link Bigodon#parse} method.
         *
         * @param {TemplateStatement} ast AST returned by {@link Bigodon#parse}.
         * @param {object?} context Context to be used when evaluating the template.
         * @param {BigodonOptions?} options Options to be used.
         * @return {Promise<string>} Promise that resolves to the rendered template.
         */
        this.run = async (ast, context, options) => {
            return await (0, runner_1.run)(ast, context, this.helpers, options);
        };
        /**
         * Compiles a template and returns a function that, when called, executes it
         *
         * @param {string} template Bigodon template to be parsed
         * @return {TemplateRunner} Function that executes the template
         */
        this.compile = (template) => {
            const ast = this.parse(template);
            return (context, options) => this.run(ast, context, options);
        };
        /**
         * Adds a helper to the Bigodon instance.
         *
         * @param {string} name Name of the helper when used in templates
         * @param {Function} helper Function to be executed when helper is called
         * @return {Bigodon} Returns the Bigodon instance for chaining
         */
        this.addHelper = (name, helper) => {
            (0, utils_1.ensure)(typeof name === 'string', 'name must be a string');
            (0, utils_1.ensure)(typeof helper === 'function', 'helper must be a function');
            this.helpers.set(name, helper);
            return this;
        };
    }
}
const defaultBigodon = new Bigodon();
/**
 * Parses a template and returns an AST representing it.
 * This can be persisted as JSON for later usage.
 *
 * @param {string} template Bigodon template to be parsed
 * @return {TemplateStatement} AST representing input template
 */
exports.parse = defaultBigodon.parse;
/**
 * Runs an AST returned by the {@link parse} method.
 *
 * @param {TemplateStatement} ast AST returned by {@link parse}.
 * @param {object?} context Context to be used when evaluating the template.
 * @param {BigodonOptions?} options Options to be used.
 * @return {Promise<string>} Promise that resolves to the rendered template.
 */
exports.run = defaultBigodon.run;
/**
 * Compiles a template and returns a function that, when called, executes it
 *
 * @param {string} template Bigodon template to be parsed
 * @return {TemplateRunner} Function that executes the template
 */
exports.compile = defaultBigodon.compile;
exports.default = Bigodon;
